#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "spcnhd:i:f:"
#define DEFAULT_HBEAT_FREQ  (8000.00)

/********************************************************
**
** Test Module: SetHeartbeat.c
**
** Purpose:     Set the heartbeat config parameters. The
**              primary one is frequency. One may also
**              specify square vs pulse, and if the
**              change should take place immediately vs.
**              begin on the next cycle.
**
**
*********************************************************/

int main (int argc, char *argv[])
{
    TPRO_BoardObj   *hnd = NULL;
    TPRO_HeartObj   heartbeat;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[MAX_DEVICE_PATH_LENGTH];
    char            *errMsg;
    char            *pgmname = argv[0];
    int             signalSelected = 0;
    int             typeSelected = 0;

    /*
     * default init the heartbeat config
     * use a pulse mode, immediate start, 8000 kHz
     */
    heartbeat.signalType = SIG_PULSE;
    heartbeat.outputType = SIG_JAM;
    heartbeat.frequency  = DEFAULT_HBEAT_FREQ;

    /*
     * parse the args
     */
    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'f' :
                heartbeat.frequency = (double)atof(optarg);
                break;
            case 's' :
                if ( signalSelected )
                {
                    usage( pgmname );
                    return (1);
                }
                heartbeat.signalType = SIG_SQUARE;
                signalSelected = 1;
                break;
            case 'p' :
                if ( signalSelected )
                {
                    usage( pgmname );
                    return (1);
                }
                heartbeat.signalType = SIG_PULSE;
                signalSelected = 1;
                break;
            case 'c' :
                if ( typeSelected )
                {
                    usage( pgmname );
                    return (1);
                }
                heartbeat.outputType = SIG_NO_JAM;
                typeSelected = 1;
                break;
            case 'n' :
                if ( typeSelected )
                {
                    usage( pgmname );
                    return (1);
                }
                heartbeat.outputType = SIG_JAM;
                typeSelected = 1;
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf_s( fullDevName, MAX_DEVICE_PATH_LENGTH, "%s%s%d", DEVICE_ROOT, devName, devIdx );


    /*
     * open the device
     */
    if ( (rc = TPRO_open(&hnd, fullDevName)) != TPRO_SUCCESS )
    {
        /* 
         * if errors, close it
         */
        printf("%s: Could not open <%s> : rc <%d>\n", 
                        pgmname, fullDevName, rc);
        return (1);
    }

    /*
     * Set the heartbeat parameters.
     */
    int done = 0;
    while ( !done )
    {
        if ( (rc = TPRO_setHeartbeat(hnd, &heartbeat)) != TPRO_SUCCESS )
        {
            char emsg[MAX_ERR_MESG_LENGTH];
            done = 1;

            switch (rc)
            {
                case TPRO_HANDLE_ERR:
                    errMsg = "TPRO_HANDLE_ERR";
                    break;
                case TPRO_COMM_ERR:
                    errMsg = "TPRO_COMM_ERR";
                    break;
                case TPRO_FREQ_ERR:
                    (void)sprintf_s( emsg, MAX_ERR_MESG_LENGTH, "TPRO_FREQ_ERR freq <%f>\n", heartbeat.frequency );
                    errMsg = emsg;
                    break;
                case TPRO_DEV_BUSY:
                    errMsg = "TPRO_DEV_BUSY waiting...";
                    done = 0;
                    break;
                default:
                    errMsg = "Unknown error";
                    break;
            }

            (void) printf("%s: <%s> %s rc: <%d>.\n", 
                        pgmname, fullDevName, errMsg, rc);

            ret = 1;

        }
        else
        {
            /*
             * success:  Print out the parameters that were set.
             */
            (void)printf("<%s> %s: Done freq <%f> mode <%s> type <%s>\n",
                        fullDevName, pgmname, heartbeat.frequency, 
                        ((heartbeat.signalType == SIG_PULSE) ? "pulse" : "square"),
                        ((heartbeat.outputType == SIG_JAM) ? "immediate" : "nextCycle"));
            done = 1;
            ret = 0;
        }

    } /* end while busy... */

    /*
     * close the handle to the board
     */
    if ( (rc = TPRO_close(hnd)) != TPRO_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n", 
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-s or -p] [-c or -n] [-h] [-d NAME] [-i INDEX] [-f frequency] \n\n", pgmName );
    (void)printf("\t-s   use square wave\n");
    (void)printf("\t-p   use pulse mode\n");
    (void)printf("\t-c   starts next cycle\n");
    (void)printf("\t-n   starts immediate\n");
    (void)printf("\t-f   frequency (e.g. 8000.0)\n");
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n\n", DEFAULT_DEV);
}
