#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:s:"

/*
 * if not supplied, create a match event to occur
 * this number of seconds past current time
 */
#define DEFAULT_MATCH_SEC   5

/* 
 * for ultra sparc .. appears that it's
 * 100 ticks per second in default configuration
 */
#define TICKS_PER_SEC       100                  

/********************************************************
**
** Test Module: SetMatch.c
**
** Purpose:     Get the current time from the board.
**              Set the starting MATCH time to be a
**              parameter-supplied number of seconds
**              into the future from the current time.
**              This assumes some other process waiting
**              for the match.
**
*********************************************************/

int main (int argc, char *argv[])
{
    TPRO_BoardObj   *hnd = NULL;
    TPRO_TimeObj    TimeObj;
    TPRO_MatchObj   MatchObj;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[MAX_DEVICE_PATH_LENGTH];
    char            *errMsg;
    char            *pgmname = argv[0];
    int             secFromCurrent = DEFAULT_MATCH_SEC;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 's' :
                secFromCurrent = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf_s( fullDevName, MAX_DEVICE_PATH_LENGTH, "%s%s%d", DEVICE_ROOT,
		devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TPRO_open(&hnd, fullDevName)) != TPRO_SUCCESS )
    {
        /* 
         * if errors, close it
         */
        printf("%s: Could not open <%s> : rc <%d>\n", 
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* until we have set the match time (board could be busy), get
     * the current time, add the specified offset in seconds and
     * request to set the match time.
     */
    int done = 0;
    while ( !done )
    {
        /* 
         * Get the TPRO/TSAT current time 
         */
        if ( (rc = TPRO_getTime(hnd, &TimeObj)) != TPRO_SUCCESS )
        {
            (void) printf("%s: <%s> Could not get current time!  rc: <%d>.\n", 
                    pgmname, fullDevName, rc);
            (void) TPRO_close( hnd );
            return (1);
        }
        else
        {
            (void)printf("\tCurrent time is: %03d:%02d:%02d:%f\n", 
                        TimeObj.days, TimeObj.hours, 
                        TimeObj.minutes, TimeObj.secsDouble);
        }

        /*
         * Set match time
         */
        MatchObj.matchType    = MATCH_TIME_START;
        MatchObj.days         = TimeObj.days;
        MatchObj.hours        = TimeObj.hours;
        MatchObj.minutes      = TimeObj.minutes;
        MatchObj.seconds      = TimeObj.secsDouble + (double) secFromCurrent ;
    
        /* handle rolling over
         */
        if (MatchObj.seconds >= 60.00)
        {
            MatchObj.seconds -= 60.00;
            MatchObj.minutes++;
            if (MatchObj.minutes >= 60)
            {
                MatchObj.minutes -= 60;
                MatchObj.hours++;
                if (MatchObj.hours >= 24)
                {
                    MatchObj.hours -= 24;
                    MatchObj.days++;
                    if (MatchObj.days >= 367)
                    {
                        MatchObj.days -= 367;
                    }
                }
            }
        }

        /* 
         * Set the TPRO/TSAT match time 
         */
        if ( (rc = TPRO_setMatchTime(hnd, &MatchObj)) != TPRO_SUCCESS )
        {
            done = 1;

            switch (rc)
            {
                case TPRO_DAY_PARM_ERR:
                    errMsg = "TPRO_DAY_PARM_ERR";
                    break;
                case TPRO_HOUR_PARM_ERR:
                    errMsg = "TPRO_HOUR_PARM_ERR";
                    break;
                case TPRO_MIN_PARM_ERR:
                    errMsg = "TPRO_MIN_PARM_ERR";
                    break;
                case TPRO_SEC_PARM_ERR:
                    errMsg = "TPRO_SEC_PARM_ERR";
                    break;
                case TPRO_DEV_BUSY:
                    errMsg = "TPRO_DEV_BUSY waiting...";
                    done = 0;
                    break;
                case TPRO_HANDLE_ERR:
                    errMsg = "TPRO_HANDLE_ERR";
                    break;
                case TPRO_COMM_ERR:
                    errMsg = "TPRO_COMM_ERR";
                    break;
                default:
                    errMsg = "Unknown error";
                break;
            }

            (void) printf("%s: <%s> Setting MatchTime %s  rc: <%d>.\n", 
                    pgmname, fullDevName, errMsg, rc);

            ret = 1;

        }
        else
        {
            (void)printf("\tMatch time is: %03d:%02d:%02d:%f\n", 
                        MatchObj.days, MatchObj.hours, 
                        MatchObj.minutes, MatchObj.seconds);

            done = 1;
            ret = 0;
        }

    } /* while !done setting match time */

    /*
     * close the handle to the board
     */
    if ( (rc = TPRO_close(hnd)) != TPRO_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n", 
                pgmname, fullDevName, rc );
        ret = 1;
    }

    (void) printf("%s: Success exiting <%s> ret: <%d>\n", 
                pgmname, fullDevName, ret );

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-s SECONDS] [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-s   # of seconds past current to match (default %d)\n", DEFAULT_MATCH_SEC);
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n\n", DEFAULT_DEV);
}

